@@ -1,3 +1,19 @@
+/*
+This program is free software: you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software Foundation,
+either version 3 of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with this program.
+If not, see <https://www.gnu.org/licenses/>.
+
+SPDX-FileCopyrightText: 2024 Arkadi Shishlov <arkadi.shishlov@gmail.com>
+SPDX-License-Identifier: GPL-3.0-or-later
+*/
+
 #include <pthread.h>
 #include <fcntl.h>
 #include <stdlib.h>
@@ -8,7 +24,7 @@
 #include <time.h>
 
 /* single file implements both G.729 and G.723.1, both IPP and Bcg729 based codecs,
-   Asterisk 1.4 to 14 */
+   Asterisk 1.4 to 20 */
 /* quite a lot of preprocessor abuse, but still better than maintaining multiple
    similar files */
 
@@ -311,8 +327,7 @@ static int g72xtolin_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
     DEBUG_FRAME_SIZE_INC;
 
     if (f->datalen == 0) {  /* Native PLC interpolation */
-        if (option_verbose > 2)
-            ast_verbose(VERBOSE_PREFIX_3 "G.729 PLC\n");
+        ast_debug(5, "G.729 PLC\n");
         if (pvt->samples + G729_SAMPLES > BUFFER_SAMPLES) {
             ast_log(LOG_WARNING, "Out of buffer space\n");
             return -1;
@@ -370,8 +385,7 @@ static int g72xtolin_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
     DEBUG_FRAME_SIZE_INC;
 
     if (f->datalen == 0) {  /* Native PLC interpolation */
-        if (option_verbose > 2)
-            ast_verbose(VERBOSE_PREFIX_3 "G.723.1 PLC\n");
+        ast_debug(5, "G.723.1 PLC\n");
         if (pvt->samples + G723_SAMPLES > BUFFER_SAMPLES) {
             ast_log(LOG_WARNING, "Out of buffer space\n");
             return -1;
@@ -462,29 +476,42 @@ static struct ast_frame *lintog72x_frameout(struct ast_trans_pvt *pvt)
     return ast_trans_frameout(pvt, datalen, samples);
 }
 
-static void g72x_destroy(struct ast_trans_pvt *pvt)
+static void g72x_print_debug_on_destroy(void)
 {
     int i;
+    /* output the sizes of frames passed to decoder */
+    if (option_debug >= 1 && frame_sizes != NULL) {
+        ast_debug(1, G72X_CODEC " frames\n");
+        ast_debug(1, "length: count\n");
+        for (i = 0; i <= DEBUG_MAX_FRAME_SIZE; ++i) {
+            if (frame_sizes[i] > 0)
+                ast_debug(1, "%6d: %d\n", i, frame_sizes[i]);
+        }
+    }
+}
+
+static void g72xtolin_destroy(struct ast_trans_pvt *pvt)
+{
     struct g72x_coder_pvt *state = pvt->pvt;
 #if !G72X_BCG729
     ippsFree(state->coder);
     ippsFree(state->scratch_mem);
 #else
-#if G72X_ASTERISK >= 160
-    ast_std_free(state->coder);
+    closeBcg729DecoderChannel(state->coder);
+#endif
+    g72x_print_debug_on_destroy();
+}
+
+static void lintog72x_destroy(struct ast_trans_pvt *pvt)
+{
+    struct g72x_coder_pvt *state = pvt->pvt;
+#if !G72X_BCG729
+    ippsFree(state->coder);
+    ippsFree(state->scratch_mem);
 #else
-    free(state->coder);
+    closeBcg729EncoderChannel(state->coder);
 #endif
-#endif
-    /* output the sizes of frames passed to decoder */
-    if (option_verbose > 2 && frame_sizes != NULL) {
-        ast_verbose(VERBOSE_PREFIX_3 G72X_CODEC " frames\n");
-        ast_verbose(VERBOSE_PREFIX_3 "length: count\n");
-        for (i = 0; i <= DEBUG_MAX_FRAME_SIZE; ++i) {
-            if (frame_sizes[i] > 0)
-                ast_verbose(VERBOSE_PREFIX_3 "%6d: %d\n", i, frame_sizes[i]);
-        }
-    }
+    g72x_print_debug_on_destroy();
 }
 
 static struct ast_translator g72xtolin = {
@@ -501,7 +528,7 @@ static struct ast_translator g72xtolin = {
 #endif
     .newpvt = g72xtolin_new,
     .framein = g72xtolin_framein,
-    .destroy = g72x_destroy,
+    .destroy = g72xtolin_destroy,
     .sample = g72xtolin_sample,
 #if G72X_CALLWEAVER
     .src_rate = 8000,
@@ -528,7 +555,7 @@ static struct ast_translator lintog72x = {
     .newpvt = lintog72x_new,
     .framein = lintog72x_framein,
     .frameout = lintog72x_frameout,
-    .destroy = g72x_destroy,
+    .destroy = lintog72x_destroy,
     .sample = lintog72x_sample,
 #if G72X_CALLWEAVER
     .src_rate = 8000,
@@ -562,8 +589,7 @@ static struct ast_translator lintog72x = {
             if (!strcasecmp(var->name, "sendrate")) {
                 rate = atoi(var->value);
                 if (rate == 53 || rate == 63) {
-                    if (option_verbose > 2)
-                        ast_verbose(VERBOSE_PREFIX_3 "G.723.1 setting sendrate to %d\n", rate);
+                    ast_debug(5, "G.723.1 setting sendrate to %d\n", rate);
                     g723_sendrate = (rate == 63) ? G723_RATE_63 : G723_RATE_53;
                 } else {
                     ast_log(LOG_ERROR, "G.723.1 sendrate must be 53 or 63\n");
@@ -595,11 +621,12 @@ static struct ast_translator lintog72x = {
         ast_free(tmp);
         ast_cli(fd, G72X_CODEC " debug disabled\n");
     } else {
-        frame_sizes = (int*)ast_malloc((DEBUG_MAX_FRAME_SIZE+1)*sizeof(int));
-        if (frame_sizes == NULL)
+        tmp = ast_malloc((DEBUG_MAX_FRAME_SIZE+1)*sizeof(int));
+        if (tmp == NULL)
             return CLI_FAILURE;
-        memset(frame_sizes, 0, (DEBUG_MAX_FRAME_SIZE+1)*sizeof(int));
+        memset(tmp, 0, (DEBUG_MAX_FRAME_SIZE+1)*sizeof(int));
         ast_cli(fd, G72X_CODEC " debug enabled\n");
+        frame_sizes = tmp;
     }
     return CLI_SUCCESS;
 }
